"""
Datamodels
----------

..note ::

    this package includes multiple alternative overlapping datamodels;
    in general a maximum of one datamodel is required for any one use case.

"""
__version__ = '0.1.0'

from oaklib.interfaces import BasicOntologyInterface

